package com.fzm.common.open.entity.openobject;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/9 14:35
 */
@Data
public class CopyrightOO {
    @NotBlank(message = "登记委托书不能为空")
    @ApiModelProperty(value = "登记委托书")
    private String registerEntrust;

    @NotBlank(message = "作品名称不能为空")
    @ApiModelProperty(value = "作品名称")
    private String opusName;

    @NotNull(message = "作品类别id不能为空")
    @ApiModelProperty(value = "作品类别id")
    private Integer opusCategoryId;

    @NotBlank(message = "作品性质不能为空")
    @ApiModelProperty(value = "作品性质")
    private String opusProperty;

    @NotBlank(message = "内容简介不能为空")
    @ApiModelProperty(value = "内容简介")
    private String contentSynopsis;

    @NotBlank(message = "创作过程不能为空")
    @ApiModelProperty(value = "创作过程")
    private String createProcess;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @NotNull(message = "作品完成日期不能为空")
    @ApiModelProperty(value = "作品完成日期，yyyy-MM-dd")
    private Date opusCompleteDate;

    @NotBlank(message = "作品完成省份不能为空")
    @ApiModelProperty(value = "作品完成省份")
    private String opusCompleteProvince;

    @NotBlank(message = "作品完成城市不能为空")
    @ApiModelProperty(value = "作品完成城市")
    private String opusCompleteCity;

    @NotNull(message = "发表状态不能为空")
    @ApiModelProperty(value = "'发表状态  0-未发表  1-已发表")
    private Integer publishState;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @ApiModelProperty(value = "首次发表日期，yyyy-MM-dd")
    private Date firstPublishDate;

    @ApiModelProperty(value = "首次发表省份")
    private String firstPublishProvince;

    @ApiModelProperty(value = "首次发表城市")
    private String firstPublishCity;

    @NotBlank(message = "权力取得方式不能为空")
    @ApiModelProperty(value = "权力取得方式")
    private String authorityAcquireMode;

    @ApiModelProperty(value = "权力取得证明材料")
    private String authorityAcquireProve;

    @NotBlank(message = "权力归属方式不能为空")
    @ApiModelProperty(value = "权力归属方式")
    private String authorityAscriptionMode;

    @ApiModelProperty(value = "权力归属证明材料的文件地址")
    private String authorityAscriptionProve;

    @NotBlank(message = "作品权力保证书不能为空")
    @ApiModelProperty(value = "作品权力保证书")
    private String opusPowerGuarantee;

    @NotEmpty(message = "作者列表不能为空")
    @ApiModelProperty(value = "作者列表")
    private List<CopyrightAuthorOO> authors;

    @NotEmpty(message = "您还未选择任何著作权人")
    @ApiModelProperty(value = "著作权人列表")
    private List<CopyrightOwnerOO> owners;

    @NotEmpty(message = "您还未勾选任何权力拥有情况")
    @ApiModelProperty("权力id集合")
    private List<Integer> authorityIds;

    @NotEmpty(message = "您还没有上传任何文件作品")
    @ApiModelProperty("文件列表")
    private List<CopyrightFileOO> files;

    @ApiModelProperty("内容")
    private String content;

    public void setOpusCompleteDate(Date opusCompleteDate) {
        if (opusCompleteDate == null) {
            this.opusCompleteDate = new Date(0);
            return;
        }
        this.opusCompleteDate = opusCompleteDate;
    }

    public void setFirstPublishDate(Date firstPublishDate) {
        if (firstPublishDate == null) {
            this.firstPublishDate = new Date(0);
            return;
        }
        this.firstPublishDate = firstPublishDate;
    }
}
