package com.fzm.common.service.impl;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.fzm.chain.simplesdk.constant.EvmTokenEnum;
import cn.fzm.chain.simplesdk.constant.TxStatusEnum;
import cn.fzm.chain.simplesdk.model.TxResult;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.*;
import com.fzm.common.entity.dto.NftPublishMsg;
import com.fzm.common.entity.vo.CollectionNftVo;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.entity.vo.NftListVo;
import com.fzm.common.entity.vo.NftVo;
import com.fzm.common.enums.AuthStatus;
import com.fzm.common.enums.PublishStatus;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.mapper.NftMapper;
import com.fzm.common.mq.NotifyPublisher;
import com.fzm.common.params.NftTransferParam;
import com.fzm.common.properties.SmsProperties;
import com.fzm.common.service.*;
import com.fzm.common.utils.ObsUtil;
import com.fzm.common.utils.RedisUtil;
import com.fzm.common.utils.SmsUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tangtuo
 * @date 2021/6/30 15:55
 */
@Slf4j
@Service
public class NftServiceImpl extends ServiceImpl<NftMapper, Nft> implements NftService {
    @Resource
    private ParaChainClient paraChainClient;
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private UserService userService;
    @Resource
    private NftMapper nftMapper;
    @Resource
    private CategoryService categoryService;
    @Resource
    private CollectionService collectionService;
    @Resource
    private NftTransferRecordService nftTransferRecordService;
    @Resource
    private SmsUtil smsUtil;
    @Resource
    private SmsProperties smsProperties;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Resource
    private ObsUtil obsUtil;
    @Resource
    private NotifyPublisher notifyPublisher;
    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Value("${chain.para.cAddr}")
    private String contractAddr;

    private static final String abi = "[{\"inputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"constructor\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"ApprovalForAll\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"values\",\"type\":\"uint256[]\"}],\"name\":\"TransferBatch\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"value\",\"type\":\"uint256\"}],\"name\":\"TransferSingle\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":false,\"internalType\":\"string\",\"name\":\"value\",\"type\":\"string\"},{\"indexed\":true,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"URI\",\"type\":\"event\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"balanceOf\",\"outputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address[]\",\"name\":\"accounts\",\"type\":\"address[]\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"}],\"name\":\"balanceOfBatch\",\"outputs\":[{\"internalType\":\"uint256[]\",\"name\":\"\",\"type\":\"uint256[]\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"burnToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"tokenIds\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"}],\"name\":\"burnTokenBatch\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"}],\"name\":\"getTokenInfo\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"}],\"name\":\"isApprovedForAll\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"mintToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"tokenIds\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"}],\"name\":\"mintTokenBatch\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeBatchTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"setApprovalForAll\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"string\",\"name\":\"tokenInfo\",\"type\":\"string\"}],\"name\":\"setTokenInfo\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"bytes4\",\"name\":\"interfaceId\",\"type\":\"bytes4\"}],\"name\":\"supportsInterface\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"transferToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"name\":\"uri\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"}]";

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer saveNft(Nft nft) {
        User user = userService.getUserByToken();
        if (!AuthStatus.SUCCESS.getStatus().equals(user.getAuthStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, "您还未实名认证,请先实名认证");
        }
        nft.setUserId(user.getId());
        nft.setPublishAddress(user.getWallet());
        save(nft);
        return nft.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void publish(Integer id) {
        Nft nft = getById(id);
        User user = userService.getUserByWallet(nft.getPublishAddress());
        String wallet = user.getWallet();
        // 获取用户的私钥
        String privkey = paraChainClient.walletDumpPrivkey(wallet);
        // 生产tokenId
        long tokenId = getTokenId();
        TreeMap<String, String> map = new TreeMap<>();
        map.put("hash", nft.getFileHash());
        map.put("publishAddress", wallet);
        map.put("author", nft.getAuthor());
        map.put("synopsis", nft.getSynopsis());
        map.put("evidencer", nft.getEvidencer());
        String str = JSONUtil.toJsonStr(map);
        String tokenInfo = this.formatTokenInfo(map);
        log.info("tokenInfo:{}", tokenInfo);
        // 发行nft
        String hash = paraChainClient.evmPublishToken(abi, contractAddr, EvmTokenEnum.ERC1155, 0L, wallet, wallet, privkey, tokenId, 1, str, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.FAILED, "nft发行失败");
        }
        String tradeHash = paraChainClient.evmSetTokenInfo(abi, contractAddr, 0L, tokenId, tokenInfo, wallet, privkey, true);

        if (StringUtils.isBlank(tradeHash)) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, "nft发行失败");
        }
        // 修改nft的发行状态为发行中
        nft.setPublishStatus(PublishStatus.PUBLISHING.getCode());
        nft.setPublishTime(new Date());
        this.updateById(nft);
        // 给mq发送一条消息,异步确认发行结果
        NftPublishMsg nftPublishMsg = new NftPublishMsg(hash, id, tokenId);
        rabbitTemplate.convertAndSend("nft-exchange", "nft.publish", nftPublishMsg);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean transfer(NftTransferParam param) {
        Nft nft = getById(param.getId());
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft详情");
        }
        // 转出人的个人信息
        User user = userService.getUserByToken();
        if (!nft.getUserId().equals(user.getId())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "您无权转让他人的nft");
        }
        // 校验短信验证码
        if (!smsUtil.validateCode(getCodeType(param.getCodeType()), user.getTelephone(), param.getCode(), param.getCodeType())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "短信验证码校验失败");
        }

        // 转让nft
        String tokenInfo = paraChainClient.evmGetTokenInfo(abi, contractAddr, nft.getTokenId(), user.getWallet());
        log.info("tokenInfo:{}", tokenInfo);

        String hash = paraChainClient.evmTransfer(abi, contractAddr, EvmTokenEnum.ERC1155, 1000000L,
                param.getReceiveWallet(), nft.getTokenId(), param.getCount(), tokenInfo, user.getWallet(), null, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "nft转让失败");
        }
        // 确认转让结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, txResult.getErrMsg().getValue());
        }
        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        if (StringUtils.isBlank(realHash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR);
        }
        // 校验接收人的个人信息
        User receiveUser = userService.getUserByWallet(param.getReceiveWallet());
        // 如果此nft已经申请版权的话,那么就不允许在一幕影链平台内部转让
        if (receiveUser != null && copyrightApplyService.existsByNftHash(nft.getNftHash())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "此nft已申请版权,不支持在一幕影链平台内部流转");
        }
        // 修改nft的拥有者用户id
        Nft transferNft = new Nft().
                setUserId(receiveUser != null ? receiveUser.getId() : 0).
                setTransferHash(realHash);
        transferNft.setId(nft.getId());
        updateById(transferNft);
        // 保存nft的转让记录

        NftTransferRecord record = new NftTransferRecord();
        record.setNftId(nft.getId()).
                setNftHash(nft.getNftHash()).
                setTransferHash(realHash).
                setFromAddress(user.getWallet()).
                setToAddress(param.getReceiveWallet());
        nftTransferRecordService.save(record);

        // 发送系统消息
        notifyPublisher.nftOutNotify(user.getId(), nft.getName());
        if (receiveUser != null) {
            notifyPublisher.nftInNotify(receiveUser.getId(), nft.getName());
        }

        return true;
    }

    @Override
    public Nft getByNftHash(String nftHash) {
        QueryWrapper<Nft> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("nft_hash", nftHash);
        return getOne(queryWrapper);
    }

    @Override
    public List<CollectionNftVo> listCopyright(Integer userId) {
        return nftMapper.listCopyright(userId);
    }

    @Override
    public Nft getByTokenId(Long tokenId) {
        QueryWrapper<Nft> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("token_id", tokenId);
        return this.getOne(queryWrapper);

    }

    @Override
    public NftVo getDetail(String nftHash) {
        Nft nft = this.getByNftHash(nftHash);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.SELECT_FAILED, "没找到此nft详情,请核对后再试");
        }

        CopyrightVo copyrightVo = copyrightApplyService.getByNftHash(nftHash);
        return new NftVo(nft, copyrightVo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateEntrust(String nftHash, Integer isEntrust) {
        Nft nft = new Nft();
        nft.setIsEntrust(isEntrust);
        QueryWrapper<Nft> wrapper = new QueryWrapper<>();
        wrapper.eq("nft_hash", nftHash);
        this.update(nft, wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Integer id) {
        Nft nft = this.getById(id);
        if (nft != null && StringUtils.isNotBlank(nft.getCover())) {
            obsUtil.delete(nft.getCover());
        }
        this.removeById(id);
    }

    @Override
    public List<Nft> getPublishingList(Integer status) {
        Integer userId = LoginUserInfo.getLoginUser().getUserId();
        QueryWrapper<Nft> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId).
                eq("publish_status", status).
                orderByDesc("publish_time");
        return this.list(queryWrapper);
    }

//    @Override
//    public void republish() {
//        List<Nft> nfts = nftMapper.get();
//        int count = 0;
//        for (Nft nft : nfts) {
//            try {
//                String OldNftHash = nft.getNftHash();
//                String newNftHash = this.republish(nft.getId());
//                copyrightApplyService.replaceNftHash(OldNftHash, newNftHash);
//                NftTransferRecord record = nftTransferRecordService.getByNftHash(OldNftHash);
//                if (record != null) {
//                    String transferHash = retransfer(nft.getId(), record.getToAddress());
//                    record.setNftHash(newNftHash);
//                    record.setTransferHash(transferHash);
//                    nftTransferRecordService.updateById(record);
//                }
//            } catch (Exception e) {
//                log.error(e.getMessage(), e);
//            }
//            log.info("已重新发行完 {} 个nft", ++count);
//        }
//    }

    private String retransfer(Integer id, String receiveAddress) {
        Nft nft = getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft详情");
        }
        User user = userService.getUserByWallet(nft.getPublishAddress());

        String tokenInfo = paraChainClient.evmGetTokenInfo(abi, contractAddr, nft.getTokenId(), user.getWallet());
        log.info("tokenInfo:{}", tokenInfo);
        // 转让nft
        String hash = paraChainClient.evmTransfer(abi, contractAddr, EvmTokenEnum.ERC1155, 1000000L,
                receiveAddress, nft.getTokenId(), 1, tokenInfo, user.getWallet(), null, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, "nft转让失败");
        }
        // 确认转让结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR, txResult.getErrMsg().getValue());
        }
        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        if (StringUtils.isBlank(realHash)) {
            throw GlobalException.newException(ResultCode.TRANSFER_ERROR);
        }
        return realHash;
    }

    private String republish(Integer id) {
        Nft nft = getById(id);
        User user = userService.getUserByWallet(nft.getPublishAddress());
        String wallet = user.getWallet();
        // 获取用户的私钥
        String privkey = paraChainClient.walletDumpPrivkey(wallet);
        // 生产tokenId
        long tokenId = nft.getTokenId();
        TreeMap<String, String> map = new TreeMap<>();
        map.put("hash", nft.getFileHash());
        map.put("publishAddress", wallet);
        map.put("author", nft.getAuthor());
        map.put("synopsis", nft.getSynopsis());
        map.put("evidencer", nft.getEvidencer());
        String str = JSONUtil.toJsonStr(map);
        String tokenInfo = this.formatTokenInfo(map);
        log.info("tokenInfo:{}", tokenInfo);
        // 发行nft
        String hash = paraChainClient.evmPublishToken(abi, contractAddr, EvmTokenEnum.ERC1155, 0L, wallet, wallet, privkey, tokenId, 1, str, true);
        if (StringUtils.isBlank(hash)) {
            throw GlobalException.newException(ResultCode.FAILED, "nft发行失败");
        }
        String tradeHash = paraChainClient.evmSetTokenInfo(abi, contractAddr, 0L, tokenId, tokenInfo, wallet, privkey, true);

        if (StringUtils.isBlank(tradeHash)) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, "nft发行失败");
        }
        // 确认交易结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
        }
        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        if (StringUtils.isBlank(realHash)) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR);
        }
        nft.setNftHash(realHash);
        nft.setPublishTime(new Date());
        nft.setPublishStatus(PublishStatus.SUSSED.getCode());
        this.updateById(nft);
        return realHash;
    }


    private String getCodeType(String type) {
        switch (type) {
            case "sms":
                return smsProperties.getTransferNftMessageCodetype();
            case "email":
                return smsProperties.getTransferNftEmailCodetype();
            case "voice":
                return smsProperties.getTransferNftVoiceCodetype();
            default:
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "短信类型【" + type + "】的传参有误,取值范围是{sms,email,voice}");
        }
    }

    @Override
    public List<Nft> list(Integer pageNum, Integer pageSize, Integer categoryId) {
        return nftMapper.list(pageNum, pageSize, categoryId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean collection(Integer id) {
        int userId = LoginUserInfo.getLoginUser().getUserId();
        String key = RedisConstant.COLLECTION_USER_PREFIX + userId;
        // 如果用户收藏的nft已经存在列表里,那么就是取消收藏
        if (redisUtil.sIsMember(key, id.toString())) {
            collectionService.deleteByUserIdAndNftId(userId, id);
            return redisUtil.sRemove(key, id.toString()) == 1;
        } else {
            TbCollection collection = new TbCollection().setNftId(id).setUserId(userId);
            try {
                collectionService.save(collection);
            } catch (DuplicateKeyException e) {
                collectionService.deleteByUserIdAndNftId(userId, id);
            }
            return redisUtil.sAdd(key, id.toString()) == 1;
        }
    }


    @Override
    public List<CollectionNftVo> getCollectionList() {
        int userId = LoginUserInfo.getLoginUser().getUserId();
        // 从redis里获取当前用户的收藏列表
        Set<String> set = redisUtil.setMembers(RedisConstant.COLLECTION_USER_PREFIX + userId);
        List<Integer> list;
        if (CollectionUtil.isEmpty(set)) {
            // redis里为空,再从mysql里查一次
            list = collectionService.getNftIdListByUserId(userId);
            if (CollectionUtil.isNotEmpty(list)) {
                // 往redis里保存一遍
                String[] array = new String[list.size()];
                for (int i = 0; i < list.size(); i++) {
                    array[i] = list.get(i).toString();
                }
                redisUtil.sAdd(RedisConstant.COLLECTION_USER_PREFIX + userId, array);
            }
        } else {
            list = set.stream().map(Integer::valueOf).collect(Collectors.toList());
        }
        // mysql和redis里都为空,直接返回空集合
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return nftMapper.getCollectionList(list);
    }

    @Override
    public List<CollectionNftVo> listCurrent(Integer categoryId, Integer userId) {
        return nftMapper.listCurrent(categoryId, userId);
    }

    @Override
    public String generateNftId(Integer categoryId) {
        Category category = categoryService.getById(categoryId);
        if (category == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此类目");
        }
        int code = RandomUtil.randomInt(1000, 10000);
        return String.format("INMVO-%s-%s%s", category.getEnglishName(), System.currentTimeMillis(), code);
    }

    @Override
    public PageInfo<NftListVo> page(Integer pageNum, Integer pageSize, Integer categoryId, String name, String telephone, Integer status, String start, String end) {
        PageHelper.startPage(pageNum, pageSize);
        DateTime startDate = null;
        DateTime endDate = null;
        if (StringUtils.isNotBlank(start)) {
            startDate = DateUtil.parse(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (StringUtils.isNotBlank(end)) {
            endDate = DateUtil.parse(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }
        List<NftListVo> list = nftMapper.page(categoryId, name, telephone, status, startDate, endDate);
        return new PageInfo<>(list);
    }

    @Override
    public Boolean top(Integer id) {
        Nft nft = getById(id);
        // 未上架的nft不能置顶
        if (SystemConstant.BOOLEAN_DATA_FALSE.equals(nft.getStatus())) {
            throw GlobalException.newException(ResultCode.FAILED, "未上架的nft不能置顶");
        }
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_TRUE);
        return updateById(nft);
    }

    @Override
    public Boolean untop(Integer id) {
        Nft nft = new Nft();
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean onShelf(Integer id) {
        Nft nft = new Nft();
        nft.setStatus(SystemConstant.BOOLEAN_DATA_TRUE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean offShelf(Integer id) {
        Nft nft = new Nft();
        nft.setStatus(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setIsTop(SystemConstant.BOOLEAN_DATA_FALSE);
        nft.setId(id);
        return updateById(nft);
    }

    @Override
    public void download(Integer id) {
        Nft nft = getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft详情");
        }
        // obsUtil.downloadFile(nft.getFileUrl(), nft.getFileName());
    }

    private long getTokenId() {
        String key = "next-tokenId";
        if (!redisUtil.hasKey(key)) {
            long second = Instant.now().getEpochSecond();
            redisUtil.set(key, String.valueOf(second));
            return second;
        }
        return redisUtil.incrBy(key, 1);
    }

    public String formatTokenInfo(TreeMap<String, String> map) {
        StringBuilder stringBuilder = new StringBuilder();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            stringBuilder.append(key).append("=").append(value).append("&");
        }
        return stringBuilder.deleteCharAt(stringBuilder.length() - 1).toString();
    }

}
