package com.fzm.portal.controller;

import cn.hutool.http.HttpStatus;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.dto.JsapiPayDto;
import com.fzm.common.enums.RefundLaunchChannel;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.WxPayService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2022/1/13 9:55
 */
@Api(tags = "微信支付")
@RestController
@RequestMapping("/wx-pay")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class WxPayController {

    private final WxPayService wxPayService;

    @Authentication
    @PostMapping("/pay/jsapi")
    @ApiOperation(value = "发起jsapi支付")
    public ResponseModel<Map<String, Object>> payH5(@RequestBody JsapiPayDto jsapiPayDto) throws Exception {
        return ResponseModel.success(wxPayService.payJsapi(jsapiPayDto));
    }

    @ApiOperation("jsapi支付回调通知")
    @PostMapping("/notify/jsapi")
    public Map<String, String> notifyJsapi(HttpServletRequest request, HttpServletResponse response) {
        Map<String, String> result = new HashMap<>();
        Boolean notify = wxPayService.notifyH5(request);
        if (notify) {
            result.put("code", "SUCCESS");
            result.put("message", "成功");
            response.setStatus(HttpStatus.HTTP_OK);
        } else {
            result.put("code", "FAILED");
            result.put("message", "系统异常");
            response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        }
        return result;
    }

    @ApiOperation("退款回调通知")
    @PostMapping("/notify/refund")
    public Map<String, String> notifyRefund(HttpServletRequest request, HttpServletResponse response) {
        Map<String, String> result = new HashMap<>();
        try {
            wxPayService.notifyRefund(request);
            result.put("code", "SUCCESS");
            result.put("message", "成功");
            response.setStatus(HttpStatus.HTTP_OK);
        } catch (Exception e) {
            result.put("code", "FAILED");
            result.put("message", "系统异常");
            response.setStatus(HttpStatus.HTTP_INTERNAL_ERROR);
        }
        return result;
    }


    @GetMapping("/refund/{orderId}")
    public ResponseModel<String> refund(@PathVariable Long orderId) {
        wxPayService.refund(orderId, RefundLaunchChannel.USER.getCode());
        return ResponseModel.success("退款成功");
    }


}
