package com.fzm.portal.schedule;

import com.fzm.common.entity.Draft;
import com.fzm.common.service.DraftService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2022/2/10 19:39
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class DraftTask {

    private final DraftService draftService;
    private final Redisson redisson;

    /**
     * 删除创建时间大于7天的草稿
     */
    @Scheduled(cron = "0 */15 * * * ?")
    public void deleteTimeoutDraft() {
        RLock lock = redisson.getLock("delete-draft");
        // 加锁,避免集群环境下多个节点同事运行此定时任务
        try {
            if (!lock.tryLock(30, TimeUnit.SECONDS)) {
                log.warn("删除超时草稿的任务正在运行中");
                return;
            }
        } catch (InterruptedException e) {
            return;
        }
        try {
            // 查询创建时间大于一周的草稿
            List<Draft> list = draftService.getTimeoutDrafts(60 * 24 * 7);
            for (Draft draft : list) {
                Integer id = draft.getId();
                log.info("即将删掉草稿,草稿id为 : {}", id);
                draftService.delete(id);
            }
        } finally {
            lock.unlock();
        }
    }
}
